print("""
Q4) Solve the following using Recursion:
• find the length of a string
• find the smallest element in a list      

def length(string):
    if string == "":
        return 0
    else:
        return 1 + length(string[1:] )
str = "Geeksfor. Geeks"

print ("length of string is :", length(str))

def find_min(list):
  if len(list) == 1:
    return list[0]
  return min(list[0], find_min(list[1:]))

listA = [9,6,1,80,9]
find_min(listA)""")


print("""
Q9 ) 9. Program to get all phone numbers of redbus.in by using web scraping and regular
  
from urllib import request
import re
reader = request.urlopen("https://www.redbus.in/info/redcare")
text = reader.read()
regex = "[0-9-]{9}[0-9-]+"
numbers = re.findall(regex, str(text), re.I)
for no in numbers:
  print(no)
o/p - 
65-31582888
2023-09-05
24595215509
  """
  )


print("""
      Q1_printpattern  Q2_printnostring     Q3_removeduplicate_andsort Q4_recursion
      Q5_copyoddlines  Q6_countwordlinechar Q7_classperson             Q8_rectangle
      Q9_redbus        Q10_phoneno          Q11_validmobno
      Q12_validgmail   Q13_birthday
      
      
      """)


print("""
Q5 ) Python program to copy odd lines from one file to another file.

def copy(from_file,to_file):
  file1 = open(from_file, "r")
  file2 = open(from_file, "w")
  lines = file1.readlines()
  for i in range(1, len(lines)):
    if (i%2 != 0):
      file2.write(lines[i])
  print("copy successful")

  file1.close()
  file2.close()

from_file = "input.txt"
to_file = "output.txt"
copy(from_file, to_file)

input.txt:            
hello 
world 
again
python snake 
zebra

output.txt:
world 
python snake 

""")


print(""" 
Q10) extract all mobile numbers present in input.txt where numbers are mixed with normal text data.

import re
file1 = open("input.txt", "r")
file2 = open("output.txt", "w")
for line in file1:
  list = re.findall('[7-9]\d{9}')
  for n in list:
    file2.write(n+"\n")
print("extracted all mobile no in output.txt")
file1.close()
file2.close()

input.txt:
hello
world
again
9284628473
python snake
0011284569
zebra

""")


print("""
Q6 ) Python program to print the number of lines, words and characters present in the given file.
      
def count_lines_word_characters(file_path):
  line_count = 0
  word_count = 0
  character_count = 0
  with open(file_path, "r") as file:
    for line in file:
      line_count +=1
      words = line.split()
      word_count += len(words)
      character_count += len(line)
  print("Number of lines :", line_count)
  print("Number of words :", word_count)
  print("Number of characters:", character_count)
  
file_path = "input.txt"
count_lines_word_characters(file_path)

input.txt:
hello 
word 
pyhton snake
elephant
cow
""")


print("""
Q8 ) class rectangle with function to find centre, area, and perimeter of a rectangle.

class Rectangle:
  def __init__(self, width, height , x_corner, y_corner):
    self.width = width
    self.height = height
    self.x_corner = x_corner
    self.y_corner = y_corner

  def area(self):
    return self.width * self.height

  def perimeter(self):
      return 2*(self.width + self.height)

  def center(self):
      center_x  =  self.x_corner + self.width/2
      center_y  =  self.y_corner + self.height/2
      return center_x, center_y

rect = Rectangle(10, 10 , 0 , 0)
center = rect.center()
area = rect.area()
peri = rect.perimeter()

print("Center :", center)
print("Area:", area)
print("Perimeter:", peri)""")


print("""
Q11) check whether the given number is a valid mobile number or not.
import re
n = input("enter number ")
m = re.fullmatch("[7-9]\d{9}",n)
if m!= None:
  print("valid mob no")
else :
  print("in valid mob no")""")


print("""
      

from  datetime import datetime, timedelta

class birthday_calc:
  def __init__(self, birthday):
    self.birthday = datetime.strptime(birthday, "%Y-%m-%d").date()

  def age(self):
    print("self.birthday:",self.birthday )
    today = datetime.today().date()
    age = today.year - self.birthday.year
    if (today.month, today.day) < (self.birthday.month, self.birthday.day):
      age -= 1
    return age

  def timetobirthday(self):
    today  = datetime.today().date()
    next_birthday = datetime(today.year, self.birthday.month, self.birthday.day ).date()

    if next_birthday<today:
      next_birthday = datetime(today.year + 1, self.birthday.month, self.birthday.day).date()

    time_till_nextbirthday = next_birthday- today
    days = time_till_nextbirthday.days
    hours , remainder = divmod(time_till_nextbirthday.seconds, 3600)
    minutes , seconds = divmod(remainder, 60)

    return days, hours , minutes, seconds

dob = input("enter birthdate (yyyy-mm-dd):")
calc = birthday_calc(dob)
age = calc.age()
print("age = ", age)

days, hours, minutes , seconds = calc.timetobirthday()
print(f"days {days } hours {hours}, minutes {minutes}, sec {seconds}")""")


print("""
Q7) Write a program that has a class Person, inherit a class Student from Person which is has a class MarksAttendance   
      
      
class Person:
  def __init__(self, usn, name, dob , gender):
    self.usn = usn
    self.name = name
    self.dob = dob
    self.gender = gender
class Student(Person):
  def __init__(self, usn, name, dob , gender ,marks, attendance):
    super().__init__(usn, name, dob, gender)
    self.marks = marks
    self.attandence = attendance
class MarksAttandence():
  def __init__ (self, marks, attandence):
    self.marks = marks
    self.attandence = attandence
    
s = Student("1nt21ai043", "XYZ", 2002-11-29," M" , 85, 98)
print("usn:", s.usn)
print("name:", s.name)
print("dob :", s.dob)
print("gender ", s.gender)
print("Marks:", s.marks)
print("Attendance:", s.attandence)""")


print("""
Q2) Given a list of strings, count and print the number of strings where the string length is 
2 or more & the 1st & last characters are same.

def count_strings(list):
  count = 0
  for s in list:
    if len(s) >=2 and s[0] == s[-1]:
      count+=1
  return count
strings = ["racecar", "apple", "civic", "pyhton", "madam"]
count = count_strings(strings)
print(f" there are {count} strings the meet the criteria")""")


print(  """
  Q1 ) Write a Program to print the following pattern.
  
  alphabet = "A"
for i in range(1,6):
  for j in range(i*2-1):
    print(alphabet, end = "")
    alphabet = chr(ord(alphabet)+1)

    if alphabet > "Z":
      alphabet = "A"
  print()
  
  """)
  


print("""
Q3 ) Write a python program to accept a sequence of whitespace separated words as input and prints the words after removing all duplicate words and sorting them alphanumerically.

def remove_dulpicate_sort(sentence):
  words = sentence.split(" ")
  unique_words = set(words)
  sorted_words = sorted(unique_words)
  return " ".join(sorted_words)

sentence = input("enter a sentence :")
result = remove_dulpicate_sort(sentence)
print(result)""")


print("""
Q12) given mail id is valid Gmail id or not.

import re
n = input("enter email id")
m = re.fullmatch("\w[a-zA-Z0-9_.]*@gmail.com", n)
if m!= None:
  print("valid gmail")
else :
  print("invalid gmail")""")